body {
  margin: 0px;
  font-family: 'Lato', sans-serif;
  overflow: hidden;
}

#container {
  padding: 0px;
  height: 100vh;
  width: 100vw;
}

#header {
  background-color: var(--design-primary-color, var(--panel-primary-color));
  color: var(--design-primary-text-color, var(-panel-on-primary-color));
  display: flex;
  align-items: center;
  padding: 10px;
}

#header-items {
  overflow: hidden;
  width: 100%;
  padding-left: 10px;
}

a.navbar-brand {
  padding-left: 10px;
  padding-right: 10px;
  display: flex;
  align-items: center;
  font-size: 1.5em;
}

.title {
  color: var(--design-primary-text-color, var(--panel-on-primary-color));
  text-decoration: none;
  text-decoration-line: none;
  text-decoration-style: initial;
  text-decoration-color: initial;
  font-weight: 400;
  font-size: 2em;
  line-height: 1.4em;
  white-space: nowrap;
}

.app-header {
  display: contents;
  padding-left: 10px;
}

.app-header a.title:hover {
  text-decoration: none;
  color: var(--design-primary-text-color, var(--panel-on-primary-color));
}

img.app-logo {
  font-size: 28px;
  height: 30px;
  max-width: inherit;
}

.pn-bar {
  width: 20px;
  height: 2px;
  background-color: var(
    --design-primary-text-color,
    var(--panel-on-primary-color)
  );
  margin: 4px 0;
}

#content {
  color: var(--text-color);
  display: flex;
  flex-direction: row;
  height: 100%;
  margin: 0px;
  overflow: hidden;
  width: 100vw;
}

#responsive-grid {
  height: 100%;
  width: 100%;
}

#sidebar-button {
  margin-left: 10px;
  margin-right: 10px;
}

#sidebar.hidden {
  border: 0px;
  margin: 0px;
  min-width: 0px;
  overflow: hidden;
  padding: 0px;
  width: 0px;
}

#sidebar {
  border-right: 1px solid var(--panel-surface-color);
  box-sizing: border-box;
  height: 100%;
  overflow: hidden auto;
  padding: 15px 10px 15px 5px;
  transition: all 0.2s cubic-bezier(0.945, 0.02, 0.27, 0.665);
  transform-origin: center left; /* Set the transformed position of sidebar to center left side. */
  overflow-y: auto;
}

#main {
  transition: all 0.2s cubic-bezier(0.945, 0.02, 0.27, 0.665);
  width: 100%;
  height: 100%;
  padding-bottom: 20px;
  overflow-y: auto;
}

#sidebarCollapse {
  background: none;
  border: none;
}

.sidenav {
  padding-top: 15px;
  overflow-y: scroll;
}

.sidenav a {
  padding: 8px 8px 8px 32px;
  text-decoration: none;
  font-size: 25px;
  color: #818181;
  display: block;
  transition: 0.3s;
}

.sidenav a:hover {
  color: #f1f1f1;
}

.sidenav .closebtn {
  position: absolute;
  top: 0;
  right: 25px;
  font-size: 36px;
  margin-left: 50px;
}

@media screen and (max-height: 450px) {
  .sidenav {
    padding-top: 15px;
  }
  .sidenav a {
    font-size: 18px;
  }
}

.nav.flex-column {
  padding-inline-start: 0px;
}

.pn-modal {
  overflow-y: scroll;
  width: 100%;
  position: absolute;
  display: none;
  top: 0;
  left: 0;
}

.pn-modal-content {
  background-color: #fefefe;
  margin: auto;
  margin-top: 25px;
  margin-bottom: 25px;
  padding: 15px 20px 20px 20px;
  border: 1px solid #888;
  width: 80% !important;
}

.pn-modal-close {
  position: absolute;
  right: 25px;
  z-index: 100;
}

.pn-modal-close:hover,
.pn-modal-close:focus {
  color: #000;
  text-decoration: none;
  cursor: pointer;
}

.pn-busy-container {
  align-items: center;
  justify-content: center;
  display: flex;
  margin-left: auto;
}

.drag-handle {
  position: absolute;
  top: 12px;
  left: 12px;
  width: 24px;
  height: 24px;
  z-index: 10000;
  background-image: url('data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCIgd2lkdGg9IjI0IiBoZWlnaHQ9IjI0Ij48cGF0aCBmaWxsPSJub25lIiBkPSJNMCAwaDI0djI0SDB6Ii8+PHBhdGggZD0iTTE2IDEzbDYuOTY0IDQuMDYyLTIuOTczLjg1IDIuMTI1IDMuNjgxLTEuNzMyIDEtMi4xMjUtMy42OC0yLjIyMyAyLjE1TDE2IDEzem0tMi03aDJ2Mmg1YTEgMSAwIDAgMSAxIDF2NGgtMnYtM0gxMHYxMGg0djJIOWExIDEgMCAwIDEtMS0xdi01SDZ2LTJoMlY5YTEgMSAwIDAgMSAxLTFoNVY2ek00IDE0djJIMnYtMmgyem0wLTR2Mkgydi0yaDJ6bTAtNHYySDJWNmgyem0wLTR2MkgyVjJoMnptNCAwdjJINlYyaDJ6bTQgMHYyaC0yVjJoMnptNCAwdjJoLTJWMmgyeiIvPjwvc3ZnPg==');
  opacity: 0;
  transition-delay: 0.5s;
  transition: 0.5s;
}

.drag-handle:hover {
  transition: 0.5s;
  opacity: 1;
}

.drag-handle:focus {
  opacity: 1;
}

.react-resizable {
  position: relative;
}
.react-resizable-handle {
  position: absolute;
  width: 20px;
  height: 20px;
  background-repeat: no-repeat;
  background-origin: content-box;
  box-sizing: border-box;
  background-image: url('data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCA2IDYiIHN0eWxlPSJiYWNrZ3JvdW5kLWNvbG9yOiNmZmZmZmYwMCIgeD0iMHB4IiB5PSIwcHgiIHdpZHRoPSI2cHgiIGhlaWdodD0iNnB4Ij48ZyBvcGFjaXR5PSIwLjMwMiI+PHBhdGggZD0iTSA2IDYgTCAwIDYgTCAwIDQuMiBMIDQgNC4yIEwgNC4yIDQuMiBMIDQuMiAwIEwgNiAwIEwgNiA2IEwgNiA2IFoiIGZpbGw9IiMwMDAwMDAiLz48L2c+PC9zdmc+');
  background-position: bottom right;
  padding: 0 3px 3px 0;
}
.react-resizable-handle-sw {
  bottom: 0;
  left: 0;
  cursor: sw-resize;
  transform: rotate(90deg);
}
.react-resizable-handle-se {
  bottom: 0;
  right: 0;
  cursor: se-resize;
}
.react-resizable-handle-nw {
  top: 0;
  left: 0;
  cursor: nw-resize;
  transform: rotate(180deg);
}
.react-resizable-handle-ne {
  top: 0;
  right: 0;
  cursor: ne-resize;
  transform: rotate(270deg);
}
.react-resizable-handle-w,
.react-resizable-handle-e {
  top: 50%;
  margin-top: -10px;
  cursor: ew-resize;
}
.react-resizable-handle-w {
  left: 0;
  transform: rotate(135deg);
}
.react-resizable-handle-e {
  right: 0;
  transform: rotate(315deg);
}
.react-resizable-handle-n,
.react-resizable-handle-s {
  left: 50%;
  margin-left: -10px;
  cursor: ns-resize;
}
.react-resizable-handle-n {
  top: 0;
  transform: rotate(225deg);
}
.react-resizable-handle-s {
  bottom: 0;
  transform: rotate(45deg);
}

.react-grid-layout {
  position: relative;
  transition: height 200ms ease;
}
.react-grid-item {
  transition: all 200ms ease;
  transition-property: left, top;
}
.react-grid-item.cssTransforms {
  transition-property: transform;
}
.react-grid-item.resizing {
  z-index: 1;
  will-change: width, height;
}

.react-grid-item.react-draggable-dragging {
  transition: none;
  z-index: 3;
  will-change: transform;
}

.react-grid-item.dropping {
  visibility: hidden;
}

.react-grid-item.react-grid-placeholder {
  background: red;
  opacity: 0.2;
  transition-duration: 100ms;
  z-index: 2;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
}

.react-grid-item > .react-resizable-handle {
  position: absolute;
  width: 20px;
  height: 20px;
  bottom: 0;
  right: 5px;
  cursor: se-resize;
}

.react-grid-item > .react-resizable-handle::after {
  content: '';
  position: absolute;
  right: 3px;
  bottom: 3px;
  width: 5px;
  height: 5px;
  border-right: 2px solid rgba(0, 0, 0, 0.4);
  border-bottom: 2px solid rgba(0, 0, 0, 0.4);
}

.react-resizable-hide > .react-resizable-handle {
  display: none;
}
